#include <stdio.h>    /* Standard input/output definitions */
#include <stdlib.h> 
#include <stdint.h>   /* Standard types */
#include <string.h>   /* String function definitions */
#include <unistd.h>   /* UNIX standard function definitions */
#include <fcntl.h>    /* File control definitions */
#include <errno.h>    /* Error number definitions */
#include <termios.h>  /* POSIX terminal control definitions */
#include <sys/ioctl.h>
#include <getopt.h>
#include <time.h>

int serialport_init(int baud);
int serialport_writebyte(int fd, uint8_t b);
int serialport_write(int fd, const char* str);
int serialport_read_until(int fd, char* buf, char until);

static int mca_handle=0;
char buf[512];
char opt[2];

void mcb_command__(int *command, int *ierr){
  int baudrate = 115200;  // default
  int stat,arg,rc;
  if(mca_handle == 0) mca_handle = serialport_init(baudrate); 
  if (mca_handle == -1){
      printf(" Erro abrindo MCA-Arduino \n");
      mca_handle =0;
      *ierr=-1;
      return;
  }
  strncpy(opt,(char *)command,1);
  //printf(" opt[0] = %c\n",opt[0]);
  if(opt[0]=='C'){//CLEAR
    sprintf(buf,"Z");
    rc = serialport_write(mca_handle, buf);
    }
  if(opt[0]=='R'){//RUN
    sprintf(buf,"R");
    rc = serialport_write(mca_handle, buf);
    }
  if(opt[0]=='S'){//STOP
    sprintf(buf,"S");
    rc = serialport_write(mca_handle, buf);
    }
}

void read_mcb__(int *nch,unsigned int vect[16384]){

union{
unsigned char ADb[4096];
unsigned int  ADi[1024];
}DAT;

int rc,n;
  for(n=0;n<1024;n++)DAT.ADi[n]=0;
  sprintf(buf,"N");
  rc = serialport_write(mca_handle, buf);
  usleep(1000000);
  n=serialport_read(mca_handle, DAT.ADb, 4096);
  DAT.ADi[0]=0;
  for(n=0;n<1024;n++)vect[n]=DAT.ADi[n];
  //for(n=0;n<1024;n++)printf(" %d %d \n",n,DAT.ADi[n]);
  *nch=1024;
  return;
}


int serialport_writebyte( int fd, uint8_t b)
{
    int n = write(fd,&b,1);
    if( n!=1)
        return -1;
    return 0;
}

int serialport_write(int fd, const char* str)
{
    int len = strlen(str);
    int n = write(fd, str, len);
    if( n!=len ) 
        return -1;
    return 0;
}

int serialport_read_until(int fd, char* buf, char until)
{
    char b[1];
    int i=0;
    do { 
        int n = read(fd, b, 1);  // read a char at a time
        if( n==-1) n = read(fd, b, 1); // couldn't read, try again
        if( n==-1) return -1;    // couldn't read
        if( n==0 ) {
            usleep( 10 * 1000 ); // wait 10 msec try again
            continue;
        }
        buf[i] = b[0]; i++;
    } while( b[0] != until );

    buf[i] = 0;  // null terminate the string
    return (i-1);
}

int serialport_read(int fd, char* buf, int nb)
{
    char b[1];
    int i=0;
    do { 
        int n = read(fd, b, 1);  // read a char at a time
        if( n==-1) n = read(fd, b, 1); // couldn't read, try again
        if( n==-1) return -1;    // couldn't read
        if( n==0 ) {
            usleep( 10 * 1000 ); // wait 10 msec try again
            continue;
        }
        buf[i] = b[0]; i++;
    } while( i<nb );

    buf[i] = 0;  // null terminate the string
    return (i-1);
}
// takes the string name of the serial port (e.g. "/dev/tty.usbserial","COM1")
// and a baud rate (bps) and connects to that port at that speed and 8N1.
// opens the port in fully raw mode so you can send binary data.
// returns valid fd, or -1 on error
int serialport_init(int baud)
{
    struct termios toptions;
    int fd;
 
    //fprintf(stderr,"init_serialport: opening port %s @ %d bps\n",
    //        serialport,baud);

    fd = open("/dev/ttyACM0", O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd == -1) {
      fd = open("/dev/ttyACM1", O_RDWR | O_NOCTTY | O_NDELAY);
      }
    if (fd == -1)  {
        printf("init_serialport: Unable to open port \n");
        return -1;
    }
    printf("Found ttyACM\n");

    memset(&toptions,0,sizeof(toptions));
    
    if (tcgetattr(fd, &toptions) < 0) {
        printf("init_serialport: Couldn't get term attributes\n");
        return -1;
    }
    speed_t brate = baud; // let you override switch below if needed
    switch(baud) {
    case 4800:   brate=B4800;   break;
    case 9600:   brate=B9600;   break;
#ifdef B14400
    case 14400:  brate=B14400;  break;
#endif
    case 19200:  brate=B19200;  break;
#ifdef B28800
    case 28800:  brate=B28800;  break;
#endif
    case 38400:  brate=B38400;  break;
    case 57600:  brate=B57600;  break;
    case 115200: brate=B115200; break;
    }
    cfsetispeed(&toptions, brate);
    cfsetospeed(&toptions, brate);

    // 8N1
    toptions.c_cflag &= ~PARENB;
    toptions.c_cflag &= ~CSTOPB;
    toptions.c_cflag &= ~CSIZE;
    toptions.c_cflag &= ~HUPCL;//disable hangup on close
    toptions.c_cflag |= CS8;
    // no flow control
    toptions.c_cflag &= ~CRTSCTS;

    toptions.c_cflag |= CREAD | CLOCAL;  // turn on READ & ignore ctrl lines
    toptions.c_iflag &= ~(IXON | IXOFF | IXANY); // turn off s/w flow ctrl

//    toptions.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG); // make raw
    toptions.c_lflag &= ~(ICANON | ECHOE | ISIG); // make raw
    toptions.c_oflag &= ~OPOST; // make raw

    // see: http://unixwiz.net/techtips/termios-vmin-vtime.html
    toptions.c_cc[VMIN]  = 0;
    toptions.c_cc[VTIME] = 20;
    
    if( tcsetattr(fd, TCSANOW, &toptions) < 0) {
        printf("init_serialport: Couldn't set term attributes\n");
        return -1;
    }

    return fd;
}



